@@ -1,10 +1,4 @@
 package Terminal::Control;
-# ----------------------------------------------------
-# Exit script if operating system Windows is detected.
-# ----------------------------------------------------
-if ($^O eq "MSWin32") {
-    exit 1;
-};
 
 # Load the basic Perl pragmas.
 use 5.030000;
@@ -19,9 +13,9 @@ use bytes;
 use Try::Catch;
 use Inline 'C' => <<'END_OF_C_CODE';
 
-#include <sys/ioctl.h>
 #include <stdio.h>
 #include <termios.h>
+#include <sys/ioctl.h>
 
 /*
  * Ref.: 
@@ -30,16 +24,43 @@ use Inline 'C' => <<'END_OF_C_CODE';
  *   https://metacpan.org/pod/perlguts
  */
 
+char* Shell() {
+    char *shell = getenv("SHELL");
+    return shell;
+};
+
+char* User() {
+    char *user = getenv("USER");
+    return user;
+};
+
+char* Logname() {
+    char *logname = getenv("LOGNAME");
+    return logname;
+};
+
+/*
+ * Function: TermType()
+ * --------------------
+ */
 char* TermType() {
     char *term_type = getenv("TERM");
     return term_type;
 };
 
+/*
+ * Function: TermColor()
+ * ---------------------
+ */
 char* TermColor() {
     char *term_color = getenv("COLORTERM");
     return term_color;
 };
 
+/*
+ * Function: TermName()
+ * --------------------
+ */
 char* TermName() {
    static char term[L_ctermid];
    const char *ptr = ctermid(term);
@@ -47,6 +68,10 @@ char* TermName() {
    return term_name;
 };
 
+/*
+ * Function: TermPath()
+ * --------------------
+ */
 char* TermPath() {
   char* tty_name = ttyname(STDIN_FILENO);
   char* term_path = (tty_name != NULL) ? tty_name : NULL;
@@ -149,7 +174,7 @@ void WinSize() {
 END_OF_C_CODE
 
 # Set the package version. 
-our $VERSION = '0.14';
+our $VERSION = '0.15';
 
 # Base class of this module.
 our @ISA = qw(Exporter);
@@ -190,7 +215,11 @@ our @EXPORT = qw(
     cursor_up
     cursor_down
     cursor_forward
-    cursor_back
+    cursor_backward
+    inuse_terminal
+    User
+    Logname
+    Shell
 );
 
 # Set the global variables.
@@ -216,6 +245,8 @@ BEGIN {
         1;
     };
     if ($@) {
+        # Print message into the terminal window.
+        print "Time::Hires is missing!\n";
         # On error use alarm.
         $mod_alarm = 'alarm';
         # Timeout is in seconds.
@@ -234,7 +265,7 @@ BEGIN {
     $is_sysph = ((eval "require 'sys/ioctl.ph'") ? 1 : 0);
     if ($is_sysph ne 1) {
         # Print message into the terminal window.
-        print "sys/ioctl.ph is missing\n";
+        print "sys/ioctl.ph is missing!\n";
         # Assign winsize to winsize_stty.
         *winsize = \&winsize_stty;
     } else {
@@ -424,8 +455,6 @@ sub ctlseqs_request {
          } catch {
              # Clean up settings.  
              system "stty -cbreak echo echonl echoke echoe isig </dev/tty >/dev/tty 2>&1";
-             # Return error code -3. 
-             #return (-3, -3);
              # Set output to error code -3. 
              ($height, $width) = (-3, -3);
          };
@@ -603,6 +632,9 @@ sub pixels {
     return ($xpix, $ypix);
 };
 
+#------------------------------------------------------------------------------# 
+# Subroutine cursor_up                                                         #
+#------------------------------------------------------------------------------# 
 sub cursor_up {
     # Set the control sequence.
     my $ctl_seq = "1A"; 
@@ -610,6 +642,9 @@ sub cursor_up {
     print "${CSI}${ctl_seq}";
 };
 
+#------------------------------------------------------------------------------# 
+# Subroutine cursor_down                                                       #
+#------------------------------------------------------------------------------# 
 sub cursor_down {
     # Set the control sequence.
     my $ctl_seq = "1B"; 
@@ -617,6 +652,9 @@ sub cursor_down {
     print "${CSI}${ctl_seq}";
 };
 
+#------------------------------------------------------------------------------# 
+# Subroutine cursor_forward                                                    #
+#------------------------------------------------------------------------------# 
 sub cursor_forward {
     # Set the control sequence.
     my $ctl_seq = "1C"; 
@@ -624,13 +662,55 @@ sub cursor_forward {
     print "${CSI}${ctl_seq}";
 };
 
-sub cursor_back {
+#------------------------------------------------------------------------------# 
+# Subroutine cursor_backward                                                       #
+#------------------------------------------------------------------------------# 
+sub cursor_backward {
     # Set the control sequence.
     my $ctl_seq = "1D"; 
     # Move the cursor back.
     print "${CSI}${ctl_seq}";
 };
 
+#------------------------------------------------------------------------------# 
+# Subroutine inuse_terminal                                                    #
+# ps -o 'cmd=' -p $(ps -o 'ppid=' -p $$)                                       #
+#------------------------------------------------------------------------------# 
+sub inuse_terminal {
+    # Get the terminal path.
+    my $term_path = TermPath();
+    # Extract the path id from the terminal path.
+    my $path_id = undef;
+    if ($term_path =~ /^.*\/dev\/(pts\/\d+)$/) {
+        $path_id = $1;
+    };
+    my $shell_path = Shell();
+    # Extract the path id from the terminal path.
+    my $shell_id = undef;
+    if ($shell_path =~ /^.*\/([a-zA-Z]+)$/) {
+        $shell_id = $1;
+    };
+    # Add square brackets for use of grep.
+    substr($path_id, 0, 0) = '['; substr($path_id, 2, 0) = ']';
+    substr($shell_id, 0, 0) = '['; substr($shell_id, 2, 0) = ']';
+    # Filter process by shell and pts.
+    my $process = `ps aux |grep $shell_id| grep $path_id`;  
+    # Split up the process by white spaces.
+    my @fields = split /\s+/, $process;
+    # Extract the PID from the array.
+    my $pid = $fields[1];
+    # Get the PPID from the command ps.
+    my $ppid = `ps -o 'ppid=' -p $pid`;
+    # Trim the PPID.  
+    $ppid =~ s/^\s+|\s+$//g;
+    # Get the terminal in use from the command ps.
+    my $terminal = `ps -o 'cmd=' -p $ppid`;
+    # Trim the terminal variable.  
+    $terminal =~ s/^\s+|\s+$//g;
+    # Return the terminal in use.
+    return $terminal;
+};
+
 1;
 
 __END__
@@ -649,7 +729,7 @@ Terminal::Control - Perl extension with methods for the control of the terminal
   clear_screen();
   reset_screen();
 
-  # Get the terminal window size using /sys/ioctl.ph or stty. 
+  # Get the terminal window size using Perl header 'sys/ioctl.ph'.
 
   ($rows, $cols, $xpix, $ypix) = winsize();
   ($rows, $cols) = chars();
@@ -662,15 +742,22 @@ Terminal::Control - Perl extension with methods for the control of the terminal
   ($rows, $cols) = screen_size_chars([$timeout]); 
   ($xpix, $ypix) = screen_size_pixels([$timeout]);
 
-  # Get the terminal window size using Inline C code using sys/ioctl.h.
+  # Get the terminal window size using C header 'sys/ioctl.h'.
 
   ($rows, $cols, $xpix, $ypix) = Terminal::Control::WinSize();
 
-  # Get and set the Cursor position on the terminal window.
+  # Get and set the cursor position on the terminal window.
 
   ($row, $col) = get_cursor_position();
   set_cursor_position($row, $col);
 
+  # Realise cursor movement on the terminal window.
+
+  cursor_up();
+  cursor_down(); 
+  cursor_forward(); 
+  cursor_back(); 
+
   # Show or hide the cursor on the terminal window.
   
   cursor_on();
@@ -681,6 +768,13 @@ Terminal::Control - Perl extension with methods for the control of the terminal
   echo_on();
   echo_off();
 
+  # Get informations about the terminal.
+
+  Terminal::Control::TermColor();
+  Terminal::Control::TermType();  
+  Terminal::Control::TermName();  
+  Terminal::Control::TermPath();  
+
 Variables in square brackets in the method call are optional. They are in the
 related method predefined.
 
@@ -688,9 +782,11 @@ related method predefined.
 
 =head2 Preface
 
-There are two areas of application for the methods presented. They are useful
-in the terminal emulator or terminal for short. However, they also work in the
-system consoles.
+There are two main areas of application for the methods presented. They
+are useful in the terminal or in the console. The terminal is a terminal
+emulator installed on current operating systems. The console is the system
+console of the operating system itself. It provides basic functions that
+may be of general interest.
 
 =head2 Methods summary
 
@@ -764,6 +860,30 @@ The methods below are sorted according to their logical relationship.
 
 =back
 
+=over 4 
+
+=item * cursor_up()
+
+=item * cursor_down()
+
+=item * cursor_forward()
+
+=item * cursor_back()
+
+=back
+
+=over 4 
+
+=item * TermColor()
+
+=item * TermType()
+
+=item * TermName()
+
+=item * TermPath()
+
+=back
+
 =head1 METHODS
 
 B<clear_screen()>
@@ -832,22 +952,22 @@ B<set_cursor_position($row, $col)>
 Set the cursor position. The method gets the current cursor position in the 
 terminal window.
 
-B<cursor_on>
+B<cursor_on()>
 
 Turn the cursor on. The cursor is enabled by using ANSI escape sequences
 in the method.
 
-B<cursor_off>
+B<cursor_off()>
 
 Turn the cursor off. The cursor is disabled by using ANSI escape sequences
 in the method.
 
-B<echo_on>
+B<echo_on()>
 
 Turn the echo on. The method uses for the moment the Shell command C<stty> for 
 turning the echo of (user) input on. 
 
-B<echo_off>
+B<echo_off()>
 
 Turn the echo off. The method uses for the moment the Shell command C<stty> for 
 turning the echo of (user) input off. 
@@ -855,6 +975,38 @@ turning the echo of (user) input off.
 Variables in square brackets in the method call are optional. They are in the
 related method predefined.
 
+B<cursor_up()>
+
+Cursor up.
+
+B<cursor_down()>
+
+Cursor down.
+
+B<cursor_forward()>
+
+Cursor forward.
+
+B<cursor_backward()>
+
+Cursor backward.
+
+B<TermColor()>
+
+Color capabilities.
+
+B<TermType()>
+
+Terminal capabilities.
+
+B<TermName()>
+
+Name of terminal.
+
+B<TermPath()>
+
+Path ot terminal.
+
 =head1 ERROR CODES
 
 The error codes of methods which are related to getting the window or sreen
@@ -866,59 +1018,121 @@ size are as follows:
 
 =head1 EXAMPLES
 
+=head2 Terminal window clear and reset
+
   # Clear the terminal window using ANSI escape sequences.
   clear_screen();
 
   # Reset the terminal window using ANSI escape sequences.
   reset_screen();
 
+=head2 Terminal window size
+
+  # Declare the variables.
+  my ($rows, $cols, $xpix, $ypix) = undef;
+
   # Get the window size calling ioctl and output the result on the screen.
-  my ($rows, $cols, $xpix, $ypix) = winsize();
+  ($rows, $cols, $xpix, $ypix) = winsize();
   printf ("%s\n%s\n%s\n%s\n", $rows, $cols, $xpix, $ypix);
 
   # Get the chars from the window size and output the result on the screen.
-  my ($rows, $cols) = chars();
+  ($rows, $cols) = chars();
   printf ("%s\n%s\n", $rows, $cols);
 
   # Get the pixels from the window size and output the result on the screen.
-  my ($xpix, $ypix) = pixels();
+  ($xpix, $ypix) = pixels();
   printf ("%s\n%s\n", $xpix, $ypix);
 
   # Set the timeout for reading from STDIN in microseconds.
   my $timeout = 1000000;
+
   # Get the window and the screen size using Xterm control sequences.
   # $timeout is optional. If not set 1000000 microseconds (1 second) are used.
-  my ($rows, $cols) = window_size_chars($timeout);
-  my ($xpix, $ypix) = windows_size_pixels($timeout);
-  my ($rows, $cols) = screen_size_chars($timeout);
-  my ($xpix, $ypix) = screen_size_pixels($timeout);
+  ($rows, $cols) = window_size_chars($timeout);
+  ($xpix, $ypix) = windows_size_pixels($timeout);
+  ($rows, $cols) = screen_size_chars($timeout);
+  ($xpix, $ypix) = screen_size_pixels($timeout);
 
   # Get the window size using Inline C code.
-  my ($rows, $cols, $xpix, $ypix) = Terminal::Control::WinSize();
+  ($rows, $cols, $xpix, $ypix) = Terminal::Control::WinSize();
   printf ("%s\n%s\n%s\n%s\n", $rows, $cols, $xpix, $ypix);
 
+=head2 Getter and setter for the cursor position
+
+  # Declare the variables.
+  my ($row, $col) = undef;
+
   # Get the cursor position in chars (not in pixels) using ANSI escape sequences.
-  my ($row, $col) = get_cursor_position();
+  ($row, $col) = get_cursor_position();
   printf ("%s\n%s\n", $row, $col);
 
   # Set the cursor position in chars (not in pixels) using ANSI escape sequences.
-  my $row = 20; 
-  my $col = 80; 
+  $row = 20; 
+  $col = 80; 
   # $row and $col are required and must be set by the user.
   set_cursor_position($row, $col);
 
+=head2 Cursor on / off
+
   # Enable visibility of the cursor using ANSI escape sequences.
   cursor_on();
 
   # Disable visibility of the cursor using ANSI escape sequences.
   cursor_off();
 
+=head2 Echo on / off
+
   # Enable echoing of commands using stty. 
   echo_on();
 
   # Disable echoing of commands using stty. 
   echo_off();
 
+=head2 Working example for cursor movement
+
+home position.The script works as follows. The terminal window is reseted.
+Then the position of the cursor is set to (20, 40) on the terminal window.
+Now the user can use the the arrow keys on the keyboard to move the cursor
+on the terminal window. Pressing 'q' quits the loop and set the cursor
+position to (1, 1) which is be the home position.
+
+  # Initialise variable $chr.
+  my $chr = '';
+
+  # Reset the terminal window.
+  reset();
+
+  # Set the initial cursor position.
+  set_cursor_position(20, 40); 
+
+  # Set some terminal attributes.
+  system("stty -echo -icanon -isig");
+
+  # Run loop until 'q' was pressed.
+  while ($chr ne "q") {
+      $chr = getc(STDIN);
+      if (ord($chr) == 65) {
+          cursor_up();
+      } elsif (ord($chr) == 66) {
+          cursor_down();
+      } elsif (ord($chr) == 67) {
+          cursor_forward();
+      } elsif (ord($chr) == 68) {
+          cursor_back();
+      };
+  };
+
+  # Reset some terminal attributes.
+  system("stty echo icanon isig");
+
+  # Reset the cursor to home.
+  set_cursor_position(1,1); 
+
+It is planned for the near future to exchange the Perl system command which
+calls stty by a C function. The basic knowledge is given, but this takes a 
+bit time to do it the right way. Such a method has to be easy to use and
+flexible in turning on and off of the attributes the same time. 
+
 =head1 VARIABLES EXPLANATION
 
 The x-direction is corresponding to window width and the y-direction is
@@ -1017,12 +1231,24 @@ control sequences>.
 
 Escape sequences used by the module:
 
-  CSI n ; m H  =>  CUP      =>  Cursor Position       =>  Moves the cursor to row n, column m 
-  CSI 6n       =>  DSR      =>  Device Status Report  =>  Reports the cursor position (CPR)
-                                                          by transmitting ESC[n;mR,
-                                                          where n is the row and m is the column
-  CSI ? 25 h   =>  DECTCEM  =>  Shows the cursor
-  CSI ? 25 l   =>  DECTCEM  =>  Hides the cursor 
+  CSI n ; m H   =>  Set Cursor Position  =>  Moves the cursor to row n and column m 
+  CSI 6n        =>  Get Cursor Position  =>  Cursor position report (CPR) by
+                                             sending ESC [ n ; m R, where
+                                             n is the row and m is the column
+
+  CSI n A       =>  Cursor Up 
+  CSI n B       =>  Cursor Down
+  CSI n C       =>  Cursor Forward
+  CSI n D       =>  Cursor Backward 
+
+  Moves the cursor n (default is 1) chars in the given direction up, down, forward or backward.
+  If the cursor is already at the one edge of the screen, this control sequence has no effect.
+
+  CSI ? 25 h    =>  Shows the cursor
+  CSI ? 25 l    =>  Hides the cursor 
+
+  CSI ? 1049 h  =>  Enable the alternative screen buffer
+  CSI ? 1049 l 	=>  Disable the alternative screen buffer 
 
 =head2 Xterm control sequences
 
@@ -1033,10 +1259,10 @@ under xterm. However, they also work with other terminal emulators.
 
 Escape sequences used by the module:
 
-  CSI 1 4 t  =>  report window size in pixels
-  CSI 1 5 t  =>  report screen size in pixels
-  CSI 1 8 t  =>  report window size in chars
-  CSI 1 9 t  =>  report screen size in chars
+  CSI 1 4 t  =>  Report the window size in pixels
+  CSI 1 5 t  =>  Report the screen size in pixels
+  CSI 1 8 t  =>  Report the window size in chars
+  CSI 1 9 t  =>  Report the screen size in chars
 
 =head1 METHOD ALIASES
 
@@ -1062,6 +1288,8 @@ Terminals tested so far with the module:
 
 =item * LXTerminal
 
+=item * mate-terminal
+
 =back
 
 =head1 LIMITATIONS
